@@ -1,22 +1,21 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2006
- *	Tama Communications Corporation
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2006,
+ *	2007 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
- * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * GNU GLOBAL is distributed in the hope that it will be useful,
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,14 +43,11 @@
 #include "global.h"
 #include "regex.h"
 #include "const.h"
-#include "filter.h"
 
 static void usage(void);
 static void help(void);
 static void setcom(int);
 int main(int, char **);
-void printtag(const char *);
-void printtag_using(const char *, const char *, int, const char *);
 void completion(const char *, const char *, const char *);
 void idutils(const char *, const char *);
 void grep(const char *, const char *);
@@ -85,14 +81,14 @@ int vflag;				/* [option]		*/
 int xflag;				/* [option]		*/
 int show_version;
 int show_help;
-int show_filter;			/* undocumented command */
 int nofilter;
 int nosource;				/* undocumented command */
 int debug;
-const char *gtags;
-int unique;
 int format;
-int passthru;
+int type;				/* path conversion type */
+char cwd[MAXPATHLEN+1];			/* current directory	*/
+char root[MAXPATHLEN+1];		/* root of source tree	*/
+char dbpath[MAXPATHLEN+1];		/* dbpath directory	*/
 
 static void
 usage(void)
@@ -109,8 +105,10 @@ help(void)
 	exit(0);
 }
 
-#define SHOW_FILTER	128
-#define RESULT		129
+#define RESULT		128
+#define SORT_FILTER     1
+#define PATH_FILTER     2
+#define BOTH_FILTER     (SORT_FILTER|PATH_FILTER)
 
 static struct option const long_options[] = {
 	{"absolute", no_argument, NULL, 'a'},
@@ -141,7 +139,6 @@ static struct option const long_options[] = {
 	{"debug", no_argument, &debug, 1},
 	{"version", no_argument, &show_version, 1},
 	{"help", no_argument, &show_help, 1},
-	{"filter", optional_argument, NULL, SHOW_FILTER},
 	{"result", required_argument, NULL, RESULT},
 	{"nosource", no_argument, &nosource, 1},
 	{ 0 }
@@ -163,9 +160,6 @@ main(int argc, char **argv)
 	int db;
 	int optchar;
 	int option_index = 0;
-	char cwd[MAXPATHLEN+1];			/* current directory	*/
-	char root[MAXPATHLEN+1];		/* root of source tree	*/
-	char dbpath[MAXPATHLEN+1];		/* dbpath directory	*/
 
 	while ((optchar = getopt_long(argc, argv, "ace:ifgGIlnoOpPqrstTuvx", long_options, &option_index)) != EOF) {
 		switch (optchar) {
@@ -254,16 +248,6 @@ main(int argc, char **argv)
 		case 'x':
 			xflag++;
 			break;
-		case SHOW_FILTER:
-			if (optarg) {
-				if (!strcmp(optarg, "sort"))
-					show_filter = SORT_FILTER;
-				else if (!strcmp(optarg, "path"))
-					show_filter = PATH_FILTER;
-			} else {
-				show_filter = BOTH_FILTER;
-			}
-			break;
 		case RESULT:
 			if (!strcmp(optarg, "ctags-x"))
 				format = FORMAT_CTAGS_X;
@@ -309,7 +293,7 @@ main(int argc, char **argv)
 	/*
 	 * only -c, -u, -P and -p allows no argument.
 	 */
-	if (!av && !show_filter) {
+	if (!av) {
 		switch (command) {
 		case 'c':
 		case 'u':
@@ -333,8 +317,6 @@ main(int argc, char **argv)
 			break;
 		}
 	}
-	if (fflag)
-		lflag = 0;
 	if (tflag)
 		xflag = 0;
 	if (nflag > 1)
@@ -365,19 +347,15 @@ main(int argc, char **argv)
 		fprintf(stdout, "%s\n", (rflag) ? root : dbpath);
 		exit(0);
 	}
-	/*
-	 * get gtags's path.
-	 */
-	gtags = usable("gtags");
-	if (!gtags)
-		die("gtags command not found.");
-	gtags = check_strdup(gtags);
 	/*
 	 * incremental update of tag files.
 	 */
 	if (uflag) {
 		STRBUF	*sb = strbuf_open(0);
+		char *gtags = usable("gtags");
 
+		if (!gtags)
+			die("gtags command not found.");
 		if (chdir(root) < 0)
 			die("cannot change directory to '%s'.", root);
 		strbuf_puts(sb, gtags);
@@ -400,35 +378,35 @@ main(int argc, char **argv)
 	}
 	/*
 	 * make local prefix.
+	 * local prefix must starts with './' and ends with '/'.
 	 */
 	if (lflag) {
-		char	*p = cwd + strlen(root);
-		STRBUF	*sb = strbuf_open(0);
-		/*
-		 * getdbpath() assure follows.
-		 * cwd != "/" and cwd includes root.
-		 */
+		STRBUF *sb = strbuf_open(0);
+
 		strbuf_putc(sb, '.');
-		if (*p)
+		if (strcmp(root, cwd) != 0) {
+			char *p = cwd + strlen(root);
+			if (*p != '/')
+				strbuf_putc(sb, '/');
 			strbuf_puts(sb, p);
+		}
 		strbuf_putc(sb, '/');
 		localprefix = check_strdup(strbuf_value(sb));
 		strbuf_close(sb);
-	} else {
-		localprefix = NULL;
+#ifdef DEBUG
+		fprintf(stderr, "root=%s\n", root);
+		fprintf(stderr, "cwd=%s\n", cwd);
+		fprintf(stderr, "localprefix=%s\n", localprefix);
+#endif
 	}
 	/*
-	 * Decide tag type.
+	 * decide tag type.
 	 */
 	db = (rflag) ? GRTAGS : ((sflag) ? GSYMS : GTAGS);
-	unique = sflag ? 1 : 0;
 	/*
 	 * decide format.
-	 * The -t and -x option are given to priority more than
-	 * the --result option.
+	 * The --result option is given to priority more than the -t and -x option.
 	 */
-	if (fflag || gflag || Pflag || Iflag || (nofilter & SORT_FILTER))
-		passthru = 1;
 	if (format == 0) {
 		if (tflag) { 			/* ctags format */
 			format = FORMAT_CTAGS;
@@ -438,42 +416,15 @@ main(int argc, char **argv)
 			format = FORMAT_PATH;
 		}
 	}
-	if (format == FORMAT_PATH)
-		unique = 1;
 	/*
-	 * setup sort filter.
+	 * decide path conversion type.
 	 */
-	setup_sortfilter(format, unique, passthru);
-	/*
-	 * setup path filter.
-	 */
-	setup_pathfilter(format, aflag ? PATH_ABSOLUTE : PATH_RELATIVE, root, cwd, dbpath);
-	/*
-	 * print external filter.
-	 */
-	if (show_filter) {
-		STRBUF  *sb;
-
-		switch (show_filter) {
-		case SORT_FILTER:
-			if (!(nofilter & SORT_FILTER))
-				fprintf(stdout, "%s\n", getsortfilter());
-			break;
-		case PATH_FILTER:
-			if (!(nofilter & PATH_FILTER))
-				fprintf(stdout, "%s\n", getpathfilter());
-			break;
-		case BOTH_FILTER:
-			sb = strbuf_open(0);
-			makefilter(sb);
-			fprintf(stdout, "%s\n", strbuf_value(sb));
-			strbuf_close(sb);
-			break;
-		default:
-			die("internal error in show_filter.");
-		}
-		exit(0);
-	}
+	if (nofilter & PATH_FILTER)
+		type = PATH_THROUGH;
+	else if (aflag)
+		type = PATH_ABSOLUTE;
+	else
+		type = PATH_RELATIVE;
 	/*
 	 * exec lid(idutils).
 	 */
@@ -519,9 +470,9 @@ main(int argc, char **argv)
 void
 completion(const char *dbpath, const char *root, const char *prefix)
 {
-	const char *p;
 	int flags = GTOP_KEY;
 	GTOP *gtop;
+	GTP *gtp;
 	int db;
 
 	flags |= GTOP_NOREGEX;
@@ -531,76 +482,70 @@ completion(const char *dbpath, const char *root, const char *prefix)
 		flags |= GTOP_PREFIX;
 	db = (sflag) ? GSYMS : GTAGS;
 	gtop = gtags_open(dbpath, root, db, GTAGS_READ);
-	for (p = gtags_first(gtop, prefix, flags); p; p = gtags_next(gtop)) {
-		fputs(p, stdout);
+	for (gtp = gtags_first(gtop, prefix, flags); gtp; gtp = gtags_next(gtop)) {
+		fputs(gtp->tag, stdout);
 		fputc('\n', stdout);
 	}
 	gtags_close(gtop);
 }
 /*
- * printtag: print a tag's line
+ * Output filter
  *
- *	i)	ctags_x	ctags -x format record
+ * (1) Old architecture (- GLOBAL-4.7.8)
+ *
+ * process1          process2       process3
+ * +=============+  +===========+  +===========+
+ * |global(write)|->|sort filter|->|path filter|->[stdout]
+ * +=============+  +===========+  +===========+
+ *
+ * (2) Recent architecture (GLOBAL-5.0 - 5.3)
+ *
+ * 1 process
+ * +===========================================+
+ * |global(write)->[sort filter]->[path filter]|->[stdout]
+ * +===========================================+
+ *
+ * (3) Current architecture (GLOBAL-5.4 -)
+ *
+ * 1 process
+ * +===========================================+
+ * |[sort filter]->global(write)->[path filter]|->[stdout]
+ * +===========================================+
+ *
+ * Sort filter is implemented in gtagsop module (libutil/gtagsop.c).
+ * Path filter is implemented in pathconvert module (libutil/pathconvert.c).
  */
-void
-printtag(const char *ctags_x)		/* virtually const */
-{
-	SPLIT ptable;
-
-	switch (format) {
-	case FORMAT_PATH:
-		split((char *)ctags_x, 4, &ptable);
-		printtag_using(
-			NULL,					/* tag */
-			ptable.part[PART_PATH].start,		/* path */
-			0,					/* line no */
-			NULL);
-		recover(&ptable);
-		break;
-	default:
-		filter_put(ctags_x);
-		break;
-	}
-}
 /*
- * printtag_using: print a tag's line with arguments
- *
- *	i)	tag	tag name
- *	i)	path	path name
- *	i)	lineno	line number
- *	i)	line	line image
+ * print number of object.
  */
 void
-printtag_using(const char *tag, const char *path, int lineno, const char *line)
+print_count(int number)
 {
-	STATIC_STRBUF(sb);
-	char edit[MAXPATHLEN];
-
-	strbuf_clear(sb);
-	/*
-	 * normalize path name.
-	 */
-	if (*path != '.') {
-		int i = 0;
-		edit[i++] = '.';
-		edit[i++] = '/';
-		while ((edit[i++] = *path++) != '\0')
-			;
-		path = edit;
+	if (format == FORMAT_PATH) {
+		switch (number) {
+		case 0:
+			fprintf(stderr, "object not found");
+			break;
+		case 1:
+			fprintf(stderr, "1 file located");
+			break;
+		default:
+			fprintf(stderr, "%d files located", number);
+			break;
+		}
+	} else {
+		switch (number) {
+		case 0:
+			fprintf(stderr, "object not found");
+			break;
+		case 1:
+			fprintf(stderr, "1 object located");
+			break;
+		default:
+			fprintf(stderr, "%d objects located", number);
+			break;
+		}
 	}
-	/*
-	 * The other format than FORMAT_PATH is always written as FORMAT_CTAGS_X.
-	 * Format conversion will be done in the following process.
-	 */
-	switch (format) {
-	case FORMAT_PATH:
-		strbuf_puts(sb, path);
-		break;
-	default:
-		strbuf_sprintf(sb, "%-16s %4d %-16s %s", tag, lineno, path, line);
-		break;
-	}
-	filter_put(strbuf_value(sb));
 }
 /*
  * idutils:  lid(idutils) pattern
@@ -612,6 +557,7 @@ void
 idutils(const char *pattern, const char *dbpath)
 {
 	FILE *ip;
+	CONVERT *cv;
 	STRBUF *ib = strbuf_open(0);
 	char edit[IDENTLEN+1];
 	const char *path, *lno, *lid;
@@ -643,25 +589,27 @@ idutils(const char *pattern, const char *dbpath)
 		fprintf(stderr, "idutils: %s\n", strbuf_value(ib));
 	if (!(ip = popen(strbuf_value(ib), "r")))
 		die("cannot execute '%s'.", strbuf_value(ib));
-	filter_open();
+	cv = convert_open(type, format, root, cwd, dbpath, stdout);
 	count = 0;
 	while ((grep = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL) {
 		p = grep;
-		/* extract filename */
+		/* extract path name */
 		path = p;
 		while (*p && *p != ':')
 			p++;
 		if ((xflag || tflag) && !*p)
 			die("invalid lid(idutils) output format. '%s'", grep);
 		*p++ = 0;
+		/* normalize path name */
+		path = makepath(".", path, NULL);
 		if (lflag) {
-			if (!locatestring(path, localprefix + 2, MATCH_AT_FIRST))
+			if (!locatestring(path, localprefix, MATCH_AT_FIRST))
 				continue;
 		}
 		count++;
 		switch (format) {
 		case FORMAT_PATH:
-			printtag_using(NULL, path, 0, NULL);
+			convert_put(cv, path);
 			break;
 		default:
 			/* extract line number */
@@ -679,26 +627,16 @@ idutils(const char *pattern, const char *dbpath)
 			/*
 			 * print out.
 			 */
-			printtag_using(edit, path, linenum, p);
+			convert_put_using(cv, edit, path, linenum, p);
 			break;
 		}
 	}
 	if (pclose(ip) < 0)
 		die("terminated abnormally.");
-	filter_close();
+	convert_close(cv);
 	strbuf_close(ib);
 	if (vflag) {
-		switch (count) {
-		case 0:
-			fprintf(stderr, "object not found");
-			break;
-		case 1:
-			fprintf(stderr, "%d object located", count);
-			break;
-		default:
-			fprintf(stderr, "%d objects located", count);
-			break;
-		}
+		print_count(count);
 		fprintf(stderr, " (using idutils index in '%s').\n", dbpath);
 	}
 }
@@ -711,6 +649,7 @@ void
 grep(const char *pattern, const char *dbpath)
 {
 	FILE *fp;
+	CONVERT *cv;
 	GFIND *gp;
 	STRBUF *ib = strbuf_open(MAXBUFLEN);
 	const char *path;
@@ -736,7 +675,7 @@ grep(const char *pattern, const char *dbpath)
 		flags |= REG_ICASE;
 	if (regcomp(&preg, pattern, flags) != 0)
 		die("invalid regular expression.");
-	filter_open();
+	cv = convert_open(type, format, root, cwd, dbpath, stdout);
 	count = 0;
 
 	gp = gfind_open(dbpath, localprefix, target);
@@ -748,29 +687,22 @@ grep(const char *pattern, const char *dbpath)
 			linenum++;
 			if (regexec(&preg, buffer, 0, 0, 0) == 0) {
 				count++;
-				printtag_using(edit, path, linenum, buffer);
-				if (format == FORMAT_PATH)
+				if (format == FORMAT_PATH) {
+					convert_put(cv, path);
 					break;
+				} else {
+					convert_put_using(cv, edit, path, linenum, buffer);
+				}
 			}
 		}
 		fclose(fp);
 	}
 	gfind_close(gp);
-	filter_close();
+	convert_close(cv);
 	strbuf_close(ib);
 	regfree(&preg);
 	if (vflag) {
-		switch (count) {
-		case 0:
-			fprintf(stderr, "object not found");
-			break;
-		case 1:
-			fprintf(stderr, "%d object located", count);
-			break;
-		default:
-			fprintf(stderr, "%d objects located", count);
-			break;
-		}
+		print_count(count);
 		fprintf(stderr, " (no index used).\n");
 	}
 }
@@ -783,6 +715,7 @@ void
 pathlist(const char *pattern, const char *dbpath)
 {
 	GFIND *gp;
+	CONVERT *cv;
 	const char *path, *p;
 	regex_t preg;
 	int count;
@@ -815,7 +748,7 @@ pathlist(const char *pattern, const char *dbpath)
 	}
 	if (!localprefix)
 		localprefix = "./";
-	filter_open();
+	cv = convert_open(type, format, root, cwd, dbpath, stdout);
 	count = 0;
 
 	gp = gfind_open(dbpath, localprefix, target);
@@ -826,23 +759,26 @@ pathlist(const char *pattern, const char *dbpath)
 		p = path + strlen(localprefix) - 1;
 		if (pattern && regexec(&preg, p, 0, 0, 0) != 0)
 			continue;
-		printtag_using("path", path, 1, " ");
+		if (format == FORMAT_PATH)
+			convert_put(cv, path);
+		else
+			convert_put_using(cv, "path", path, 1, " ");
 		count++;
 	}
 	gfind_close(gp);
-	filter_close();
+	convert_close(cv);
 	if (pattern)
 		regfree(&preg);
 	if (vflag) {
 		switch (count) {
 		case 0:
-			fprintf(stderr, "path not found");
+			fprintf(stderr, "file not found");
 			break;
 		case 1:
-			fprintf(stderr, "%d path located", count);
+			fprintf(stderr, "%d file located", count);
 			break;
 		default:
-			fprintf(stderr, "%d paths located", count);
+			fprintf(stderr, "%d files located", count);
 			break;
 		}
 		fprintf(stderr, " (using '%s').\n", makepath(dbpath, dbname(GPATH), NULL));
@@ -861,8 +797,11 @@ pathlist(const char *pattern, const char *dbpath)
 void
 parsefile(int argc, char **argv, const char *cwd, const char *root, const char *dbpath, int db)
 {
+	CONVERT *cv;
 	char rootdir[MAXPATHLEN+1];
 	char buf[MAXPATHLEN+1], *path;
+	const char *basename;
+	STATIC_STRBUF(dir);
 	int count = 0;
 	STRBUF *comline = strbuf_open(0);
 	STRBUF *path_list = strbuf_open(MAXPATHLEN);
@@ -889,7 +828,7 @@ parsefile(int argc, char **argv, const char *cwd, const char *root, const char *
 	 */
 	if (!getconfs(dbname(db), comline))
 		die("cannot get parser for %s.", dbname(db));
-	filter_open();
+	cv = convert_open(type, format, root, cwd, dbpath, stdout);
 	if (gpath_open(dbpath, 0) < 0)
 		die("GPATH not found.");
 	/*
@@ -911,11 +850,26 @@ parsefile(int argc, char **argv, const char *cwd, const char *root, const char *
 		/*
 		 * convert path into relative from root directory of source tree.
 		 */
-		path = realpath(av, buf);
+		strbuf_clear(dir);
+		basename = locatestring(av, "/", MATCH_LAST);
+		if (basename != NULL) {
+			strbuf_nputs(dir, av, basename - av);
+			basename++;
+		} else {
+			strbuf_putc(dir, '.');
+			basename = av;
+		}
+		/*
+		 * Expand symlink only in directory part.
+		 */
+		path = realpath(strbuf_value(dir), buf);
 		if (path == NULL)
-			die("realpath(%s, buf) failed. (errno=%d).", av, errno);
+			die("realpath(%s, buf) failed. (errno=%d).", strbuf_value(dir), errno);
 		if (!isabspath(path))
 			die("realpath(3) is not compatible with BSD version.");
+		if (strcmp(path, "/") != 0)
+			strcat(path, "/");
+		strcat(path, basename);
 		/*
 		 * Remove the root part of path and insert './'.
 		 *      rootdir  /a/b/
@@ -927,6 +881,7 @@ parsefile(int argc, char **argv, const char *cwd, const char *root, const char *
 				fprintf(stderr, "'%s' is out of source tree.\n", buf);
 			continue;
 		}
+		/* normalize path name */
 		path -= 2;
 		*path = '.';
 		if (!gpath_path2fid(path, NULL)) {
@@ -934,6 +889,8 @@ parsefile(int argc, char **argv, const char *cwd, const char *root, const char *
 				fprintf(stderr, "'%s' not found in GPATH.\n", path);
 			continue;
 		}
+		if (lflag && !locatestring(path, localprefix, MATCH_AT_FIRST))
+			continue;
 		/*
 		 * Add a path to the path list.
 		 */
@@ -945,9 +902,27 @@ parsefile(int argc, char **argv, const char *cwd, const char *root, const char *
 	if (chdir(root) < 0)
 		die("cannot move to '%s' directory.", root);
 	xp = xargs_open_with_strbuf(strbuf_value(comline), 0, path_list);
-	while ((ctags_x = xargs_read(xp)) != NULL) {
-		printtag(ctags_x);
-		count++;
+	if (format == FORMAT_PATH) {
+		SPLIT ptable;
+		char curpath[MAXPATHLEN+1];
+
+		curpath[0] = '\0';
+		while ((ctags_x = xargs_read(xp)) != NULL) {
+			if (split((char *)ctags_x, 4, &ptable) < 4) {
+				recover(&ptable);
+				die("too small number of parts.\n'%s'", ctags_x);
+			}
+			if (strcmp(curpath, ptable.part[PART_PATH].start)) {
+				strlimcpy(curpath, ptable.part[PART_PATH].start, sizeof(curpath));
+				convert_put(cv, curpath);
+				count++;
+			}
+		}
+	} else {
+		while ((ctags_x = xargs_read(xp)) != NULL) {
+			convert_put(cv, ctags_x);
+			count++;
+		}
 	}
 	xargs_close(xp);
 	if (chdir(cwd) < 0)
@@ -956,21 +931,11 @@ parsefile(int argc, char **argv, const char *cwd, const char *root, const char *
 	 * Settlement
 	 */
 	gpath_close();
-	filter_close();
+	convert_close(cv);
 	strbuf_close(comline);
 	strbuf_close(path_list);
 	if (vflag) {
-		switch (count) {
-		case 0:
-			fprintf(stderr, "object not found");
-			break;
-		case 1:
-			fprintf(stderr, "%d object located", count);
-			break;
-		default:
-			fprintf(stderr, "%d objects located", count);
-			break;
-		}
+		print_count(count);
 		fprintf(stderr, " (no index used).\n");
 	}
 }
@@ -984,25 +949,39 @@ parsefile(int argc, char **argv, const char *cwd, const char *root, const char *
  *	i)	db		GTAGS,GRTAGS,GSYMS
  *	r)			count of output lines
  */
+/* get next number and seek to the next character */
+#define GET_NEXT_NUMBER(p) do {                                                 \
+                if (!isdigit(*p))                                              \
+                        p++;                                                    \
+                for (n = 0; isdigit(*p); p++)                                  \
+                        n = n * 10 + (*p - '0');                                \
+        } while (0)
 int
 search(const char *pattern, const char *root, const char *cwd, const char *dbpath, int db)
 {
-	const char *ctags_x;
+	CONVERT *cv;
 	int count = 0;
 	GTOP *gtop;
+	GTP *gtp;
 	int flags = 0;
-	STRBUF *sb = NULL;
+	STRBUF *sb = NULL, *ib = NULL;
+	char curpath[MAXPATHLEN+1], curtag[IDENTLEN+1];
+	FILE *fp = NULL;
+	const char *src = "";
+	int lineno, last_lineno;
 
+	lineno = last_lineno = 0;
+	curpath[0] = curtag[0] = '\0';
 	/*
 	 * open tag file.
 	 */
 	gtop = gtags_open(dbpath, root, db, GTAGS_READ);
-	filter_open();
+	cv = convert_open(type, format, root, cwd, dbpath, stdout);
 	/*
 	 * search through tag file.
 	 */
-	if (nosource)
-		flags |= GTOP_NOSOURCE;
+	if (nofilter & SORT_FILTER)
+		flags |= GTOP_NOSORT;
 	if (iflag) {
 		if (!isregex(pattern)) {
 			sb = strbuf_open(0);
@@ -1015,20 +994,162 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 	}
 	if (Gflag)
 		flags |= GTOP_BASICREGEX;
-	for (ctags_x = gtags_first(gtop, pattern, flags); ctags_x; ctags_x = gtags_next(gtop)) {
-		if (lflag) {
-			const char *q;
-			/* locate start point of a path */
-			q = locatestring(ctags_x, "./", MATCH_FIRST);
-			if (!locatestring(q, localprefix, MATCH_AT_FIRST))
-				continue;
+	if (format == FORMAT_PATH)
+		flags |= GTOP_PATH;
+	if (gtop->format & GTAGS_COMPACT)
+		ib = strbuf_open(0);
+	for (gtp = gtags_first(gtop, pattern, flags); gtp; gtp = gtags_next(gtop)) {
+		if (lflag && !locatestring(gtp->path, localprefix, MATCH_AT_FIRST))
+			continue;
+		if (format == FORMAT_PATH) {
+			convert_put(cv, gtp->path);
+			count++;
+		} else if (gtop->format & GTAGS_COMPACT) {
+			/*
+			 *                    a          b
+			 * tagline = <file id> <tag name> <line no>,...
+			 */
+			char *p = (char *)gtp->tagline;
+			const char *tagname;
+			int n = 0;
+
+			while (*p != ' ')
+				p++;
+			*p++ = '\0';			/* a */
+			tagname = p;
+			while (*p != ' ')
+				p++;
+			*p++ = '\0';			/* b */
+			/*
+			 * Reopen or rewind source file.
+			 */
+			if (!nosource) {
+				if (strcmp(gtp->path, curpath) != 0) {
+					if (curpath[0] != '\0' && fp != NULL)
+						fclose(fp);
+					strlimcpy(curtag, tagname, sizeof(curtag));
+					strlimcpy(curpath, gtp->path, sizeof(curpath));
+					/*
+					 * Use absolute path name to support GTAGSROOT
+					 * environment variable.
+					 */
+					fp = fopen(makepath(root, curpath, NULL), "r");
+					if (fp == NULL)
+						warning("source file '%s' is not available.", curpath);
+					last_lineno = lineno = 0;
+				} else if (strcmp(gtp->tag, curtag) != 0) {
+					strlimcpy(curtag, gtp->tag, sizeof(curtag));
+					if (atoi(p) < last_lineno && fp != NULL) {
+						rewind(fp);
+						lineno = 0;
+					}
+					last_lineno = 0;
+				}
+			}
+			/*
+			 * Unfold compact format.
+			 *
+			 * If GTAGS_COMPLINE flag is set, each line number is expressed as
+			 * the difference from the previous line number except for the head.
+			 * Please see flush_pool() in libutil/gtagsop.c for the details.
+			 */
+			if (!isdigit(*p))
+				die("illegal compact format.");
+			if (gtop->format & GTAGS_COMPLINE) {
+				int last = 0, cont = 0;
+
+				while (*p || cont > 0) {
+					if (cont > 0) {
+						n = last + 1;
+						if (n > cont) {
+							cont = 0;
+							continue;
+						}
+					} else if (isdigit(*p)) {
+						GET_NEXT_NUMBER(p);
+					}  else if (*p == '-') {
+						GET_NEXT_NUMBER(p);
+						cont = n + last;
+						n = last + 1;
+					} else if (*p == ',') {
+						GET_NEXT_NUMBER(p);
+						n += last;
+					}
+					if (last_lineno != n && fp) {
+						while (lineno < n) {
+							if (!(src = strbuf_fgets(ib, fp, STRBUF_NOCRLF)))
+								die("unexpected end of file. '%s: %d/%d'", gtp->path, lineno, n);
+							lineno++;
+						}
+					}
+					if (gtop->format & GTAGS_COMPNAME)
+						tagname = (char *)uncompress(tagname, gtp->tag);
+					convert_put_using(cv, tagname, gtp->path, n, src);
+					count++;
+					last_lineno = last = n;
+				}
+			} else {
+				while (*p) {
+					for (n = 0; isdigit(*p); p++)
+						n = n * 10 + *p - '0';
+					if (*p == ',')
+						p++;
+					if (last_lineno == n)
+						continue;
+					if (last_lineno != n && fp) {
+						while (lineno < n) {
+							if (!(src = strbuf_fgets(ib, fp, STRBUF_NOCRLF)))
+								die("unexpected end of file. '%s: %d/%d'", gtp->path, lineno, n);
+							lineno++;
+						}
+					}
+					if (gtop->format & GTAGS_COMPNAME)
+						tagname = (char *)uncompress(tagname, gtp->tag);
+					convert_put_using(cv, tagname, gtp->path, n, src);
+					count++;
+					last_lineno = n;
+				}
+			}
+		} else {
+			/*
+			 *                    a          b         c
+			 * tagline = <file id> <tag name> <line no> <line image>
+			 */
+			char *p = (char *)gtp->tagline;
+			char namebuf[IDENTLEN+1];
+			const char *tagname, *image;
+
+			while (*p != ' ')
+				p++;
+			*p++ = '\0';			/* a */
+			tagname = p;
+			while (*p != ' ')
+				p++;
+			*p++ = '\0';			/* b */
+			if (gtop->format & GTAGS_COMPNAME) {
+				strlimcpy(namebuf, (char *)uncompress(tagname, gtp->tag), sizeof(namebuf));
+				tagname = namebuf;
+			}
+			if (nosource) {
+				image = " ";
+			} else {
+				while (*p != ' ')
+					p++;
+				image = p + 1;		/* c + 1 */
+				if (gtop->format & GTAGS_COMPRESS)
+					image = (char *)uncompress(image, gtp->tag);
+			}
+			convert_put_using(cv, tagname, gtp->path, gtp->lineno, image);
+			count++;
 		}
-		printtag(ctags_x);
-		count++;
 	}
-	filter_close();
+	convert_close(cv);
 	if (sb)
 		strbuf_close(sb);
+	if (ib)
+		strbuf_close(ib);
+	if (fp)
+		fclose(fp);
 	gtags_close(gtop);
 	return count;
 }
@@ -1072,11 +1193,6 @@ tagsearch(const char *pattern, const char *cwd, const char *root, const char *db
 				continue;
 			if (!test("f", makepath(libdbpath, dbname(db), NULL)))
 				continue;
-
-			/*
-			 * reconstruct path filter
-			 */
-			setup_pathfilter(format, aflag ? PATH_ABSOLUTE : PATH_RELATIVE, libdir, cwd, libdbpath);
 			/*
 			 * search again
 			 */
@@ -1091,17 +1207,7 @@ tagsearch(const char *pattern, const char *cwd, const char *root, const char *db
 		strbuf_close(sb);
 	}
 	if (vflag) {
-		switch (total) {
-		case 0:
-			fprintf(stderr, "'%s' not found", pattern);
-			break;
-		case 1:
-			fprintf(stderr, "%d object located", total);
-			break;
-		default:
-			fprintf(stderr, "%d objects located", total);
-			break;
-		}
+		print_count(total);
 		if (!Tflag)
 			fprintf(stderr, " (using '%s')", makepath(dbpath, dbname(db), NULL));
 		fputs(".\n", stderr);
