@@ -4,19 +4,18 @@
  *
  * This file is part of GNU GLOBAL.
  *
- * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * GNU GLOBAL is distributed in the hope that it will be useful,
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -84,12 +83,6 @@ const char *file_list;
  */
 int do_path;
 int convert_type = PATH_RELATIVE;
-int format = FORMAT_PATH;
-/*
- * Sort filter
- */
-int do_sort;
-int unique;
 
 int extractmethod;
 int total;
@@ -131,19 +124,15 @@ static struct option const long_options[] = {
 	 */
 	/* flag value */
 	{"debug", no_argument, &debug, 1},
-	{"sort", no_argument, &do_sort, 1},
-	{"unique", no_argument, &unique, 1},
 	{"version", no_argument, &show_version, 1},
 	{"help", no_argument, &show_help, 1},
 
 	/* accept value */
 #define OPT_CONFIG		128
-#define OPT_FORMAT		129
-#define OPT_GTAGSCONF		130
-#define OPT_GTAGSLABEL		131
-#define OPT_PATH		132
+#define OPT_GTAGSCONF		129
+#define OPT_GTAGSLABEL		130
+#define OPT_PATH		131
 	{"config", optional_argument, NULL, OPT_CONFIG},
-	{"format", required_argument, NULL, OPT_FORMAT},
 	{"gtagsconf", required_argument, NULL, OPT_GTAGSCONF},
 	{"gtagslabel", required_argument, NULL, OPT_GTAGSLABEL},
 	{"path", required_argument, NULL, OPT_PATH},
@@ -179,20 +168,6 @@ main(int argc, char **argv)
 			if (optarg)
 				config_name = optarg;
 			break;
-		case OPT_FORMAT:
-			if (!strcmp("ctags", optarg))
-				format = FORMAT_CTAGS;
-			else if (!strcmp("path", optarg))
-				format = FORMAT_PATH;
-			else if (!strcmp("grep", optarg))
-				format = FORMAT_GREP;
-			else if (!strcmp("cscope", optarg))
-				format = FORMAT_CSCOPE;
-			else if (!strcmp("ctags-x", optarg))
-				format = FORMAT_CTAGS_X;
-			else
-				die("Unknown format type.");
-			break;
 		case OPT_GTAGSCONF:
 			gtagsconf = optarg;
 			break;
@@ -270,26 +245,6 @@ main(int argc, char **argv)
 		else
 			fprintf(stdout, "%s\n", getconfline());
 		exit(0);
-	} else if (do_sort) {
-		/*
-		 * This is the main body of sort filter.
-		 *
-		 * - Requirement -
-		 * 1. input must be one of these format:
-		 *    0: ctags -x format
-		 *    1: ctags format
-		 *    2: path name
-		 * 2. input must be sorted in alphabetical order by tag name
-		 *    if it is ctags [-x] format.
-		 */
-		STRBUF *ib = strbuf_open(MAXBUFLEN);
-		TAGSORT *ts = tagsort_open(output, format, unique, 0);
-
-		while (strbuf_fgets(ib, stdin, STRBUF_NOCRLF) != NULL)
-			tagsort_put(ts, strbuf_value(ib));
-		tagsort_close(ts);
-		strbuf_close(ib);
-		exit(0);
 	} else if (do_path) {
 		/*
 		 * This is the main body of path filter.
@@ -311,7 +266,7 @@ main(int argc, char **argv)
 		 * main      22 /prj/xxx/libc/func.c   main(argc, argv)\n
 		 */
 		STRBUF *ib = strbuf_open(MAXBUFLEN);
-		CONVERT *cv = convert_open(convert_type, format, argv[0], argv[1], argv[2], stdout);
+		CONVERT *cv = convert_open(convert_type, FORMAT_CTAGS_X, argv[0], argv[1], argv[2], stdout);
 		char *ctags_x;
 
 		if (argc < 3)
@@ -496,7 +451,7 @@ incremental(const char *dbpath, const char *root)
 	IDSET *deleteset, *findset;
 	int updated = 0;
 	const char *path;
-	int i, limit;
+	unsigned int id, limit;
 
 	if (vflag) {
 		fprintf(stderr, " Tag found in '%s'.\n", dbpath);
@@ -566,11 +521,11 @@ incremental(const char *dbpath, const char *root)
 	 * make delete list.
 	 */
 	limit = gpath_nextkey();
-	for (i = 1; i < limit; i++) {
+	for (id = 1; id < limit; id++) {
 		char fid[32];
 		int type;
 
-		snprintf(fid, sizeof(fid), "%d", i);
+		snprintf(fid, sizeof(fid), "%d", id);
 		/*
 		 * This is a hole of GPATH. The hole increases if the deletion
 		 * and the addition are repeated.
@@ -582,12 +537,12 @@ incremental(const char *dbpath, const char *root)
 		 * assuming that it does not exist in the file system.
 		 */
 		if (type == GPATH_OTHER) {
-			if (!idset_contains(findset, i) || !test("f", path) || test("b", path))
+			if (!idset_contains(findset, id) || !test("f", path) || test("b", path))
 				strbuf_puts0(deletelist, path);
 		} else {
-			if (!idset_contains(findset, i) || !test("f", path)) {
+			if (!idset_contains(findset, id) || !test("f", path)) {
 				strbuf_puts0(deletelist, path);
-				idset_add(deleteset, i);
+				idset_add(deleteset, id);
 			}
 		}
 	}
@@ -595,7 +550,7 @@ incremental(const char *dbpath, const char *root)
 	/*
 	 * execute updating.
 	 */
-	if (deleteset->max + strbuf_getlen(addlist)) {
+	if (!idset_empty(deleteset) || strbuf_getlen(addlist) > 0) {
 		int db;
 
 		for (db = GTAGS; db < GTAGLIM; db++) {
@@ -641,11 +596,7 @@ incremental(const char *dbpath, const char *root)
 		 * because they may have no definitions.
 		 */
 		for (db = GTAGS; db < GTAGLIM; db++)
-#ifdef HAVE_UTIMES
-			utimes(makepath(dbpath, dbname(db), NULL), NULL);
-#else
 			utime(makepath(dbpath, dbname(db), NULL), NULL);
-#endif /* HAVE_UTIMES */
 	}
 	if (vflag) {
 		if (updated)
@@ -706,20 +657,18 @@ updatetags(const char *dbpath, const char *root, IDSET *deleteset, STRBUF *addli
 		char fid[32];
 		const char *path;
 		int total = idset_count(deleteset);
-		int i;
+		unsigned int id;
 
 		seqno = 1;
-		for (i = 0; i < deleteset->max; i++) {
-			if (idset_contains(deleteset, i)) {
-				snprintf(fid, sizeof(fid), "%d", i);
-				path = gpath_fid2path(fid, NULL);
-				if (path == NULL)
-					die("GPATH is corrupted.");
-				fprintf(stderr, " [%d/%d] deleting tags of %s\n", seqno++, total, path + 2);
-			}
+		for (id = idset_first(deleteset); id != END_OF_ID; id = idset_next(deleteset)) {
+			snprintf(fid, sizeof(fid), "%d", id);
+			path = gpath_fid2path(fid, NULL);
+			if (path == NULL)
+				die("GPATH is corrupted.");
+			fprintf(stderr, " [%d/%d] deleting tags of %s\n", seqno++, total, path + 2);
 		}
 	}
-	if (deleteset->max > 0)
+	if (!idset_empty(deleteset))
 		gtags_delete(gtop, deleteset);
 	gtop->flags = 0;
 	if (extractmethod)
